# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-09-20 09:45
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cost_tracking', '0025_increase_price_decimal_places'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='priceestimate',
            name='limit',
        ),
        migrations.RemoveField(
            model_name='priceestimate',
            name='threshold',
        ),
    ]
